@@ -1,11 +1,12 @@
 """Test utils for ansible-lint."""
+from __future__ import annotations
 
 import os
 import shutil
 import subprocess
 import sys
 import tempfile
-from typing import TYPE_CHECKING, Any, Dict, List, Optional
+from typing import TYPE_CHECKING, Any
 
 from ansiblelint.app import get_app
 from ansiblelint.rules import RulesCollection
@@ -14,15 +15,15 @@ if TYPE_CHECKING:
     # https://github.com/PyCQA/pylint/issues/3240
     # pylint: disable=unsubscriptable-object
     CompletedProcess = subprocess.CompletedProcess[Any]
+    from ansiblelint.errors import MatchError  # noqa: E402
 else:
     CompletedProcess = subprocess.CompletedProcess
 
 # Emulate command line execution initialization as without it Ansible module
 # would be loaded with incomplete module/role/collection list.
-app = get_app()
+app = get_app(offline=True)
 
 # pylint: disable=wrong-import-position
-from ansiblelint.errors import MatchError  # noqa: E402
 from ansiblelint.runner import Runner  # noqa: E402
 
 
@@ -33,11 +34,15 @@ class RunFromText:
         """Initialize a RunFromText instance with rules collection."""
         self.collection = collection
 
-    def _call_runner(self, path: str) -> List["MatchError"]:
+    def _call_runner(self, path: str) -> list[MatchError]:
         runner = Runner(path, rules=self.collection)
         return runner.run()
 
-    def run_playbook(self, playbook_text: str) -> List[MatchError]:
+    def run(self, filename: str) -> list[MatchError]:
+        """Lints received filename."""
+        return self._call_runner(filename)
+
+    def run_playbook(self, playbook_text: str) -> list[MatchError]:
         """Lints received text as a playbook."""
         with tempfile.NamedTemporaryFile(
             mode="w", suffix=".yml", prefix="playbook"
@@ -47,7 +52,7 @@ class RunFromText:
             results = self._call_runner(fh.name)
         return results
 
-    def run_role_tasks_main(self, tasks_main_text: str) -> List[MatchError]:
+    def run_role_tasks_main(self, tasks_main_text: str) -> list[MatchError]:
         """Lints received text as tasks."""
         role_path = tempfile.mkdtemp(prefix="role_")
         tasks_path = os.path.join(role_path, "tasks")
@@ -58,7 +63,7 @@ class RunFromText:
         shutil.rmtree(role_path)
         return results
 
-    def run_role_meta_main(self, meta_main_text: str) -> List[MatchError]:
+    def run_role_meta_main(self, meta_main_text: str) -> list[MatchError]:
         """Lints received text as meta."""
         role_path = tempfile.mkdtemp(prefix="role_")
         meta_path = os.path.join(role_path, "meta")
@@ -69,19 +74,35 @@ class RunFromText:
         shutil.rmtree(role_path)
         return results
 
+    def run_role_defaults_main(self, defaults_main_text: str) -> list[MatchError]:
+        """Lints received text as vars file in defaults."""
+        role_path = tempfile.mkdtemp(prefix="role_")
+        defaults_path = os.path.join(role_path, "defaults")
+        os.makedirs(defaults_path)
+        with open(os.path.join(defaults_path, "main.yml"), "w", encoding="utf-8") as fh:
+            fh.write(defaults_main_text)
+        results = self._call_runner(role_path)
+        shutil.rmtree(role_path)
+        return results
+
 
 def run_ansible_lint(
     *argv: str,
-    cwd: Optional[str] = None,
-    executable: Optional[str] = None,
-    env: Optional[Dict[str, str]] = None
+    cwd: str | None = None,
+    executable: str | None = None,
+    env: dict[str, str] | None = None,
+    offline: bool = True,
 ) -> CompletedProcess:
     """Run ansible-lint on a given path and returns its output."""
+    args = [*argv]
+    if offline:
+        args.insert(0, "--offline")
+
     if not executable:
         executable = sys.executable
-        args = [sys.executable, "-m", "ansiblelint", *argv]
+        args = [sys.executable, "-m", "ansiblelint", *args]
     else:
-        args = [executable, *argv]
+        args = [executable, *args]
 
     # It is not safe to pass entire env for testing as other tests would
     # pollute the env, causing weird behaviors, so we pass only a safe list of
@@ -108,11 +129,10 @@ def run_ansible_lint(
 
     return subprocess.run(
         args,
-        stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE,
+        capture_output=True,
         shell=False,  # needed when command is a list
         check=False,
         cwd=cwd,
         env=_env,
-        universal_newlines=True,
+        text=True,
     )
